package com.oscar.base;

import com.github.pagehelper.PageInfo;
import com.oscar.base.vo.ResultVO;
import com.oscar.module.sign.TbClockIn;
import com.oscar.sys.menu.SysMenu;
import freemarker.template.TemplateException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.List;

public abstract class AbstractController<S extends AbstractService, T extends AbstractEntity> {
    // spring 泛型依赖注入
    @Autowired
    protected S service;

    @GetMapping("entity")
    public ResultVO entity (T entity) {
        return success(service.get(entity));
    }

    @GetMapping("page")
    public PageInfo<T> pageListX (HttpServletRequest request, T entity) {
        return pageList(request, entity);
    }

    protected PageInfo<T> pageList (HttpServletRequest request, T entity) {
        return service.pageList(request, entity);
    }

    @GetMapping("list")
    public ResultVO findListX (T entity) {
        return findList(entity);
    }

    protected ResultVO findList (T entity) {
        return success(service.findList(entity));
    }

    @PostMapping
    public ResultVO saveX (@RequestBody T entity) {
        return save(entity);
    }

    protected ResultVO save (T entity) {
        service.save(entity);
        return success();
    }

    @DeleteMapping
    public ResultVO deleteX (@RequestBody T entity) {
        return delete(entity);
    }

    protected ResultVO delete (T entity) {
        service.delete(entity);
        return success();
    }

    @PostMapping("approve")
    public ResultVO approve (@RequestBody T entity) {
        service.save(entity);
        return success();
    }

    protected ResultVO success (List<T> list) {
        return ResultVO.success().setList(list);
    }

    protected ResultVO success (T entity) {
        return ResultVO.success().set(entity);
    }

    protected ResultVO success () {
        return ResultVO.success();
    }

    protected ResultVO success (Object data) {
        return ResultVO.success(data);
    }

    protected ResultVO error (Exception e) {
        return ResultVO.error(e.getMessage());
    }

    protected ResultVO error (String message) {
        return ResultVO.error(message);
    }

    protected ResultVO warn (String message) {
        return ResultVO.warn(message);
    }

    protected void setResponseAsAttachment (HttpServletResponse response, String fileName) {
        if (fileName == null) {
            System.out.println("filename is null!");
            return;
        } else {
            try {
                //                fileName = URLEncoder.encode(fileName, "utf-8");
                fileName = new String(fileName.getBytes("gb2312"), "ISO8859-1");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
        }
        response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
        response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
    }

}
