package com.oscar.base;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;
import java.util.Optional;

//    ExampleMatcher matcher = ExampleMatcher.matching()
//                    .withMatcher("userName", match -> match.endsWith())
//                    .withMatcher("模糊匹配userName", match -> match.contains())
//                    .withMatcher("前缀匹配userName", match -> match.startsWith());
public class AbstractService<D extends AbstractMapper, T extends AbstractEntity<T>> {
    @Autowired
    protected D dao;

    public void save(T entity) {
        if (entity.getCreateTime() == null) {
            entity.setCreateTime(new Date());
        }

        dao.save(entity);
    }

    public void delete(T entity) {
        dao.delete(entity);
    }

    public void batchInsert(List<T> list) {
        dao.saveAll(list);
    }

    public void batchDelete(List<T> list) {
        dao.deleteAll(list);
    }

    public int count(T entity) {
        return (int) dao.count();
    }

    public List<T> findList() {
        return dao.findAll();
    }

    public List<T> findList(T entity) {
        return dao.findAll(Example.of(entity));
    }

    public Optional<T> get(T entity) {
        List<T> all = dao.findAll(Example.of(entity));
        T result = all.isEmpty() ? entity : all.get(0);
        return Optional.of(result);
    }

    public PageInfo<T> pageList(HttpServletRequest request, T entity) {
        return pageList(request, entity);
    }

    public PageInfo<T> pageList(HttpServletRequest request, T entity, String sortId, String order) {
        Integer pageNum = Integer.parseInt(request.getParameter("pageNum"));
        Integer pageSize = Integer.parseInt(request.getParameter("pageSize"));

        PageRequest pageRequest;
        if (sortId == null) {
            pageRequest = PageRequest.of(pageNum, pageSize);
        } else {
            Sort.Direction direction = ("desc".equalsIgnoreCase(order)) ? Sort.Direction.DESC : Sort.Direction.ASC;
            Sort sort = new Sort(direction, sortId);
            pageRequest = PageRequest.of(pageNum, pageSize, sort);
        }
        Page page = dao.findAll(pageRequest);

        PageInfo pageInfo = new PageInfo();
        pageInfo.setTotal(page.getTotalElements());
        pageInfo.setPageNum(page.getNumber());
        pageInfo.setPageSize(page.getNumberOfElements());
        pageInfo.setList(page.getContent());
        pageInfo.setHasNextPage(page.hasNext());
        pageInfo.setHasPreviousPage(page.hasPrevious());
        pageInfo.setIsFirstPage(page.isFirst());
        pageInfo.setIsLastPage(page.isLast());

        return pageInfo;
    }

    public PageInfo<T> pageList1(HttpServletRequest request, T entity) {
        Integer pageNum = Integer.parseInt(request.getParameter("pageNum"));
        Integer pageSize = Integer.parseInt(request.getParameter("pageSize"));
        PageHelper.startPage(pageNum, pageSize);
        List<T> list = findList(entity);
        return new PageInfo<>(list);
    }
}
