package com.oscar.base.config;

import com.oscar.base.vo.ResultVO;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.ClientHttpRequestFactory;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.util.HashMap;
import java.util.Map;

/**
 * Bean注册
 */
@Configuration
public class HttpConfig {

    /**
     * Http client的工厂，支持jdk原生的，Netty httpclient，OkHttp3等
     */
    @Bean
    public ClientHttpRequestFactory simpleClientHttpRequestFactory() {
        // JDK原生
        SimpleClientHttpRequestFactory factory = new SimpleClientHttpRequestFactory();
        // OkHttp3ClientHttpRequestFactory factory = new OkHttp3ClientHttpRequestFactory();//需要加依赖
        factory.setReadTimeout(5000);    // 5000ms
        factory.setConnectTimeout(5000); // 5000ms
        return factory;
    }

    @Bean
    public RestTemplate restTemplate(ClientHttpRequestFactory factory) {
        return new RestTemplate(factory);
    }

    public void testAPI() {
        RestTemplate restTemplate = new RestTemplate();
        Map<String, String> map = new HashMap();
        map.put("start", "1");
        map.put("page", "5");
        ResultVO result = restTemplate.getForObject("http://fantj.top/notice/list/", ResultVO.class, map);

        String url = "http://47.xxx.xxx.96/register/checkEmail";
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        MultiValueMap<String, String> multiValueMap = new LinkedMultiValueMap<>();
        multiValueMap.add("email", "844072586@qq.com");

        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(multiValueMap, headers);
        ResponseEntity<String> response = restTemplate.postForEntity(url, request, String.class);
        System.out.println(response.getBody());
    }
}
