package com.oscar.base.config;

import cn.hutool.log.StaticLog;
import com.zaxxer.hikari.HikariDataSource;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;
import org.springframework.jdbc.datasource.init.DatabasePopulatorUtils;
import org.springframework.jdbc.datasource.init.ResourceDatabasePopulator;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.net.URI;
import java.net.URISyntaxException;
import java.sql.*;

@Configuration
@Order(1)
public class InitDatabaseOnStart {
    @Resource
    private HikariDataSource dataSource;
    @Value("${config.database.ddl}")
    private org.springframework.core.io.Resource resource;

    @PostConstruct
    public void init() throws ClassNotFoundException, URISyntaxException {
        String driver = dataSource.getDriverClassName();
        String username = dataSource.getUsername();
        String password = dataSource.getPassword();
        String url = dataSource.getJdbcUrl();

        URI uri = new URI(url.replace("jdbc:", ""));
        String path = uri.getPath();
        String database = path.replace("/", "");

        Class.forName(driver);

        String connectUrl = url.split(path)[0];

        // jdbc连接db
        try (Connection connection = DriverManager.getConnection(connectUrl, username, password);
             Statement statement = connection.createStatement()) {

            // 查询database的个数
            ResultSet rs = statement.executeQuery("SELECT COUNT(*) from information_schema.SCHEMATA where SCHEMA_NAME = '" + database + "'");

            if (rs.next()) {
                if (rs.getInt(1) == 0) {
                    StaticLog.info("{} not exists, start create it.", database);
                    statement.executeUpdate("CREATE DATABASE IF NOT EXISTS `" + database + "` DEFAULT CHARACTER SET = `utf8` COLLATE `utf8_general_ci`;");
                    runScript();
                } else {
                    StaticLog.info("{} exists.", database);
                }

            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    /**
     * run data.sql
     * see org.springframework.boot.autoconfigure.jdbc.DataSourceInitializer
     */
    private void runScript() {
        ResourceDatabasePopulator populator = new ResourceDatabasePopulator();
        populator.setContinueOnError(false);
        // populator.setSeparator(";");
        populator.setSqlScriptEncoding("utf-8");
        populator.addScript(resource);
        DatabasePopulatorUtils.execute(populator, dataSource);
        StaticLog.info("data.sql run successfully!");
    }
}
