package com.oscar.base.util;

import com.aliyuncs.CommonRequest;
import com.aliyuncs.CommonResponse;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.exceptions.ServerException;
import com.aliyuncs.http.MethodType;
import com.aliyuncs.profile.DefaultProfile;
import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;

import java.util.concurrent.TimeUnit;

public class SmsUtils {
    //phone, code
    private static Cache<String,String> cache = CacheBuilder.newBuilder().maximumSize(30000)
            .expireAfterAccess(5, TimeUnit.MINUTES)
            .build();

    /**
     * 验证码是否正确
     */
    public static boolean isValid(String phone,String code){
		if(code != null && code.equals(cache.getIfPresent(phone))){ //正确
            cache.invalidate(phone);
            return true;
        }else{
            return false;
        }
    }

    /**
     * 生成验证码
     */
    public static int randomCodeGenerate(String phone){
        int code;
        if(cache.getIfPresent(phone) == null){ //send
            code = (int) ((Math.random() * 9 + 1) * 100000) ;
            cache.put(phone,String.valueOf(code));
        }else{
            code = -1;
        }
        return code;
    }

    /**
     * 发送验证码
     */
    public static void sendSmsCode(String phone, String code){
        DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", "LTAI1DPfICZJNjA6", "<accessSecret>");
        IAcsClient client = new DefaultAcsClient(profile);

        CommonRequest request = new CommonRequest();
        request.setMethod(MethodType.POST);
        request.setDomain("dysmsapi.aliyuncs.com");
        request.setVersion("2017-05-25");
        request.setAction("SendSms");
        request.putQueryParameter("RegionId", "cn-hangzhou");
        request.putQueryParameter("PhoneNumbers", phone);
        request.putQueryParameter("SignName", "百涉校园");
        request.putQueryParameter("TemplateCode", "SMS_145780329");
        request.putQueryParameter("TemplateParam", "{\"code\":\""+code+"\"}");
        try {
            CommonResponse response = client.getCommonResponse(request);
            System.out.println(response.getData());
        } catch (ServerException e) {
            e.printStackTrace();
        } catch (ClientException e) {
            e.printStackTrace();
        }
    }
}
