package com.oscar.base.util;

import org.apache.mahout.cf.taste.common.TasteException;
import org.apache.mahout.cf.taste.impl.common.FastByIDMap;
import org.apache.mahout.cf.taste.impl.common.LongPrimitiveIterator;
import org.apache.mahout.cf.taste.impl.model.GenericDataModel;
import org.apache.mahout.cf.taste.impl.model.GenericUserPreferenceArray;
import org.apache.mahout.cf.taste.impl.model.file.FileDataModel;
import org.apache.mahout.cf.taste.impl.neighborhood.NearestNUserNeighborhood;
import org.apache.mahout.cf.taste.impl.recommender.GenericUserBasedRecommender;
import org.apache.mahout.cf.taste.impl.similarity.PearsonCorrelationSimilarity;
import org.apache.mahout.cf.taste.model.DataModel;
import org.apache.mahout.cf.taste.model.PreferenceArray;
import org.apache.mahout.cf.taste.neighborhood.UserNeighborhood;
import org.apache.mahout.cf.taste.recommender.RecommendedItem;
import org.apache.mahout.cf.taste.recommender.Recommender;
import org.apache.mahout.cf.taste.similarity.UserSimilarity;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 推荐思路
 * 1、读取用户-物品-评分数据，转换成推荐数据模型
 * 2、基于用户相似度计算用户N-最近邻
 * 3、使用推荐引擎推荐物品
 */
public class UserBasedRecommendation {
    /*public static Set<Integer> recommend(List<TbOrder> orders, Integer targetUserId) {
        Map<Integer,List<TbOrder>> map = new HashMap<>(); //用户,订单列表
        for (TbOrder order : orders){
            List<TbOrder> array = map.get(order.getUserId());
            if(array == null) array = new ArrayList<>();
            array.add(order);
            map.put(order.getUserId(), array);
        }

        FastByIDMap<PreferenceArray> preMap = new FastByIDMap<>();
        for(Integer userId : map.keySet()){
            final List<TbOrder> list = map.get(userId);
            PreferenceArray preferenceArray = new GenericUserPreferenceArray(list.size());
            //            System.out.println(userId+"  order size= "+list.size());
            for(int i=0;i<list.size();i++){
                final TbOrder order = list.get(i);
                if(order.getScore() != null){ //针对已经完成评分的订单
                    preferenceArray.setUserID(i, userId);
                    preferenceArray.setItemID(i,order.getProductId());
                    preferenceArray.setValue(i, order.getScore());
                }
            }

            preMap.put(userId, preferenceArray);
        }

        DataModel dataModel = new GenericDataModel(preMap);
        Set<Integer> productIds = new HashSet<>();

        try {
            UserSimilarity userSimilarity = new PearsonCorrelationSimilarity(dataModel);
            UserNeighborhood userNeighborhood = new NearestNUserNeighborhood(2, userSimilarity, dataModel);
            Recommender recommender = new GenericUserBasedRecommender(dataModel, userNeighborhood, userSimilarity);
            List<RecommendedItem> recommendedItems = recommender.recommend(targetUserId, 2);


            for (RecommendedItem item : recommendedItems) {
                System.out.println("recommend:"+item.getItemID());
                productIds.add(new Long(item.getItemID()).intValue());
            }
        }catch (Exception e){
            e.printStackTrace();
        }

        return productIds;
    }*/

    private static List<Integer> recommend(Integer userId) throws TasteException {
        //        存储用户偏好
        FastByIDMap<PreferenceArray> preMap = new FastByIDMap<PreferenceArray>();
        //构造用户1的偏好
        PreferenceArray user1 = new GenericUserPreferenceArray(10);
        user1.setUserID(0, 1L);
        user1.setItemID(0, 101L);
        user1.setValue(0, 3.0f);

        user1.setUserID(1, 1L);
        user1.setItemID(1, 101L);
        user1.setValue(1, 3.0f);

        user1.setUserID(2, 1L);
        user1.setItemID(2, 101L);
        user1.setValue(2, 3.0f);

        preMap.put(1L,user1);

        DataModel dataModel = new GenericDataModel(preMap);
        UserSimilarity userSimilarity = new PearsonCorrelationSimilarity(dataModel);
        UserNeighborhood userNeighborhood = new NearestNUserNeighborhood(2, userSimilarity, dataModel);
        Recommender recommender = new GenericUserBasedRecommender(dataModel, userNeighborhood, userSimilarity);
        List<RecommendedItem> recommendedItems = recommender.recommend(userId, 2);

        List<Integer> productIds = new ArrayList<>();
        for (RecommendedItem item : recommendedItems) {
            productIds.add(new Long(item.getItemID()).intValue());
        }
        return productIds;
    }

    public static void main (String[] args) {

        DataModel dataModel;
        try {
            dataModel = new FileDataModel(new File("D:/alice.txt"));//用户-物品-评分数据文件
            /**
             * 用户相似度算法
             */
            //余弦相似度
            // UserSimilarity userSimilarity= new UncenteredCosineSimilarity(dataModel);
            //欧几里得
            // UserSimilarity userSimilarity= new EuclideanDistanceSimilarity(dataModel);
            //皮尔森相似度
            UserSimilarity userSimilarity = new PearsonCorrelationSimilarity(dataModel);

            //相邻用户相似度（用户的2-最近邻）
            UserNeighborhood userNeighborhood = new NearestNUserNeighborhood(2, userSimilarity, dataModel);
            //定义推荐引擎:user-based
            Recommender recommender = new GenericUserBasedRecommender(dataModel, userNeighborhood, userSimilarity);

            //从数据模型中获取所有用户ID迭代器
            LongPrimitiveIterator usersIterator = dataModel.getUserIDs();
            while (usersIterator.hasNext()) {
                System.out.println("================================================");
                //用户ID
                long userID = usersIterator.nextLong();
                //用户ID
                LongPrimitiveIterator otherusersIterator = dataModel.getUserIDs();
                //遍历用户ID，计算任何两个用户的相似度
                while (otherusersIterator.hasNext()) {
                    Long otherUserID = otherusersIterator.nextLong();
                    System.out.println("用户 " + userID + " 与用户 " + otherUserID + " 的相似度为："
                            + userSimilarity.userSimilarity(userID, otherUserID));
                }
                //userID的N-最近邻
                long[] userN = userNeighborhood.getUserNeighborhood(userID);

                //向用户userID的推荐物品，最多推荐两个
                List<RecommendedItem> recommendedItems = recommender.recommend(userID, 2);
                System.out.println("用户 " + userID + " 的2-最近邻是 " + Arrays.toString(userN));
                if (recommendedItems.size() > 0) {
                    for (RecommendedItem item : recommendedItems) {
                        System.out.println("推荐的物品" + item.getItemID() + "预测评分是 " + item.getValue());
                    }
                } else {
                    System.out.println("无任何物品推荐");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
