package com.oscar.gen;

import cn.hutool.core.util.StrUtil;
import com.oscar.base.AbstractController;
import com.oscar.base.util.StringUtils;
import com.oscar.base.vo.ResultVO;
import com.oscar.gen.android.AndroidGen;
import com.oscar.gen.express.ExpressGen;
import com.oscar.gen.springboot.SpringBootGen;
import com.oscar.gen.vue.VueGen;
import com.oscar.gen.vux.VuxGen;
import com.oscar.gen.wechat.WechatGen;
import com.oscar.sys.SystemMapper;
import com.oscar.sys.menu.SysMenu;
import com.oscar.sys.menu.SysMenuService;
import com.oscar.sys.rolemenu.SysRoleMenu;
import com.oscar.sys.rolemenu.SysRoleMenuService;
import freemarker.template.TemplateException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/code/gen")
@CrossOrigin
public class CodeGenController extends AbstractController<CodeGenService, CodeGen> {
    @Autowired
    SysMenuService sysMenuService;
    @Autowired
    SysRoleMenuService sysRoleMenuService;
    @Autowired
    SystemMapper systemMapper;

    @GetMapping("schema")
    public List<Column> schemaList () {
        return service.schemaList();
    }

    @GetMapping("table")
    public List<Column> tableList (Column param) {
        return service.tableList(param);
    }

/**
     * 返回只是tb_ 开头的表名
     */

    @GetMapping("tb/name")
    public List<String> tbNameList (Column param) {
        final List<Column> columns = service.tableList(param);
        return columns.stream().map(column -> column.getTableName() + "|" + column.getTableComment()).filter(name -> (name.startsWith(param.getTableClassify()))).collect(Collectors.toList());
    }

    @PostMapping("table")
    public ResultVO genTable (@RequestBody CodeGen codeGen) {
        String sql = "CREATE TABLE " + codeGen.getTableName() + " (" + codeGen.getSql() + " PRIMARY KEY (id) )";
        sql += "ENGINE=INNODB AUTO_INCREMENT=1000 DEFAULT CHARSET=utf8 COMMENT='" + codeGen.getSchema() + "'";
        systemMapper.exec(sql);
        return ResultVO.success();
    }


/**
     * 在sys_menu中创建菜单
     */

    private void menuInit (String name, String route) {
        final List<SysMenu> list = sysMenuService.findList(new SysMenu());
        boolean created = false;
        int currentId = 900;
        int parentId = 9;
        for (SysMenu menu : list) {
            if (menu.getHref().equals(route)) {
                created = true;
            }
            if (menu.getParentId() == parentId && menu.getId() > currentId) {
                currentId = menu.getId();
            }
        }

        if (!created) {
            final SysMenu sysMenu = new SysMenu();
            sysMenu.setId(currentId + 1);
            sysMenu.setParentId(parentId);
            sysMenu.setName(name);
            sysMenu.setHref(route);
            sysMenu.setEnabled("on");
            sysMenu.setIcon("el-icon-document");
            sysMenuService.save(sysMenu);

            SysRoleMenu roleMenu = new SysRoleMenu();
            roleMenu.setRoleId(1);
            roleMenu.setMenuId(sysMenu.getId());
            sysRoleMenuService.save(roleMenu);
        }
    }


/**
     * 生成模板主函数
     */

    @Override
    public ResultVO save (CodeGen codeGen) {
        String t = codeGen.getTableName(); //tb_oscar_info|奥斯卡最佳男主角
        final String[] split = t.split("\\|");

        //todo no use
        if(split.length == 2)
            menuInit(split[1], "/" + split[0]);

        codeGen.setTableName(split[0]);
        codeGen.setClassName(StringUtils.toCapitalizeCamelCase(codeGen.getTableName()));
        codeGen.setEntityVar(StrUtil.toCamelCase(codeGen.getTableName()));
        codeGen.setRequestUri(parseUri(codeGen.getTableName()));
        codeGen.setColumns(service.fetchColumns(codeGen));
        codeGen.setPageStyle(new PageStyle(codeGen.getCss()));
        ///////////////////////////////////////////////////
        for (Column column : codeGen.getColumns()) {
            if ("userId".equals(column.getField())) { //userid字段
                codeGen.setHasUserIdColumn(true);
            }
            if ("state".equals(column.getField())) { //审核字段
                codeGen.setHasStateColumn(true);
            }
        }

        try {
            SpringBootGen.gen(codeGen);
            ExpressGen.gen(codeGen);
            VueGen.gen(codeGen);
            VuxGen.gen(codeGen);
            WechatGen.gen(codeGen);
            AndroidGen.gen(codeGen);
        }catch (Exception e){
            e.printStackTrace();
        }
        return ResultVO.success();
    }

    private String parseUri (String tableName) {
        String[] split = tableName.split("_");
        StringBuilder uri = new StringBuilder();
        for (String seg : split) {
            uri.append("/").append(seg);
        }
        return uri.toString();
    }
}
