package com.oscar.gen;

import com.google.common.io.Files;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import freemarker.template.TemplateExceptionHandler;
import org.springframework.core.io.DefaultResourceLoader;
import org.springframework.core.io.Resource;

import java.io.*;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class FreeMarkerUtils {
    static Configuration cfg;

    static {
        cfg = new Configuration(Configuration.VERSION_2_3_28);
        Resource path = new DefaultResourceLoader().getResource("classpath:/templates");
        try {
            cfg.setDirectoryForTemplateLoading(path.getFile());
        } catch (IOException e) {
            e.printStackTrace();
        }
        cfg.setDefaultEncoding("UTF-8");
        cfg.setTemplateExceptionHandler(TemplateExceptionHandler.RETHROW_HANDLER);
    }


    public static void gen (String output, String fileName, Object model, String template, String suffix) throws IOException, TemplateException {
        File outputFile = new File(output);
        if (!outputFile.exists()) {
            outputFile.mkdirs();
        }

        if(!output.endsWith("/"))
            output += "/";

        final String pathname = output + fileName + suffix;
        System.out.println("生成文件:" + pathname);
        OutputStream fos = new FileOutputStream(new File(pathname));
        Writer out = new OutputStreamWriter(fos);

        Template temp = cfg.getTemplate(template);
        temp.process(model, out);
        fos.flush();
        fos.close();
    }

    public static void main (String[] args) throws IOException, TemplateException {
        //        Configuration cfg = new Configuration(Configuration.VERSION_2_3_28);
        //        Resource path = new DefaultResourceLoader().getResource("classpath:/templates");
        //        cfg.setDirectoryForTemplateLoading(path.getFile());
        //        cfg.setDefaultEncoding("UTF-8");
        //        cfg.setTemplateExceptionHandler(TemplateExceptionHandler.RETHROW_HANDLER);
        //
        //        Template temp = cfg.getTemplate("mapper.ftl");
        //
        //        Map<String, Object> root = getDao();
        //        Writer out = new OutputStreamWriter(System.out);
        //        temp.process(root, out);
    }

    public static Map<String, Object> getDao () {
        Map<String, Object> root = new HashMap<String, Object>();

        root.put("packageName", "com.oscar.code.generater");
        root.put("className", "CodeGen");
        return root;
    }

    /**
     * 生成vue router/index.js
     * @param router     tb_oscar_info
     * @param fileName   tb_oscar_info
     */
    private static void refreshRouter (String router, CodeGen codeGen, String fileName, String routerJs) throws IOException {
        String moduleName = codeGen.getModuleName();
        String title = codeGen.getVue().getTitle();


        String filePath =  codeGen.getVue().getFilePath();// D:/IdeaProjects/private_job/oscar/web/src/page/examples/oscar/
        String relativePath = filePath.split("src/page/")[1];//examples/oscar/
        String path = "@/page/" + relativePath;
        if(!path.endsWith("/")) path += "/";
        String json = "        {path: '/" + router + "', name: '"+title+"', component: r => require.ensure([], () => r(require('" + path + fileName + "')), '" + moduleName + "')},";

        //write to src/router/index.js
        File file = new File(routerJs);
        final List<String> list = Files.readLines(file, Charset.forName("utf-8"));
        StringBuilder out = new StringBuilder();

        for (String s : list) {
            if (s.equals(json)) {
                return;
            }
        }

        for (String s : list) {
            out.append(s).append("\n");
            if (s.trim().startsWith("// business route start")) {
                out.append(json).append("\n");
            }
        }

        Files.write(out.toString().getBytes(), file);
    }

    public static void refreshRouter (CodeGen codeGen, String vueComponent) {
        String routerJs = codeGen.getProjectPath().replaceAll("/java/", "/web/src/router/index.js");
        try {
            refreshRouter(vueComponent, codeGen, vueComponent, routerJs);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    public static void refreshMobileRouter (String router, String moduleName, String fileName, String routerJs) throws IOException {
        String path = "@/page/" + moduleName + "/mobile/";
        String json = "  {path: '/m/" + router + "', component: r => require.ensure([], () => r(require('" + path + fileName + "')), 'mobile_" + moduleName + "')},";

        //write to src/router/index.js
        File file = new File(routerJs);
        final List<String> list = Files.readLines(file, Charset.forName("utf-8"));
        StringBuilder out = new StringBuilder();

        for (String s : list) {
            if (s.equals(json)) {
                return;
            }
        }

        for (String s : list) {
            out.append(s).append("\n");
            if (s.trim().startsWith("// mobile router start")) {
                out.append(json).append("\n");
            }
        }

        Files.write(out.toString().getBytes(), file);
    }
}
