package com.oscar.gen;

import com.oscar.base.AbstractController;
import com.oscar.base.config.ConfigSysProperties;
import com.oscar.base.util.FileUtils;
import com.oscar.base.vo.ResultVO;
import com.oscar.sys.SystemMapper;
import com.oscar.sys.menu.SysMenuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.io.File;

@RestController
@RequestMapping("/code/module")
public class ModuleClearController {
    @Autowired
    SysMenuService sysMenuService;
    @Autowired
    SystemMapper systemMapper;
    @Autowired
    ConfigSysProperties configSysProperties;

    @Value("${spring.application.name}")
    private String applicationName;

    @Autowired
    public ModuleClearController (SysMenuService sysMenuService) {
        this.sysMenuService = sysMenuService;
    }

    private void removeWechatFiles (String path) {
        File file = new File(getWechatRoot() + path);
        if (file.isDirectory()) {
            FileUtils.deleteDir(file);
        } else {
            file.delete();
        }
    }

    /**
     * @param path src目录下
     */
    private void removeVueFiles (String dir, String path) {
        File file = new File(getVueRoot() + dir + path);
        if (file.isDirectory()) {
            FileUtils.deleteDir(file);
        } else {
            file.delete();
        }
    }

    private void removeVueFiles (String path) {
        removeVueFiles("/src", path);
    }

    /**
     * @param path src/main目录下
     */
    private void removeJavaFiles (String path) {
        File file = new File(javaRoot() + "/src/main" + path);
        if (file.isDirectory()) {
            FileUtils.deleteDir(file);
        } else {
            file.delete();
        }
    }

    /**
     * @param path src/test/java目录下
     */
    private void removeJavaTextFiles (String path) {
        File file = new File(javaRoot() + "/src/test/java" + path);
        if (file.isDirectory()) {
            FileUtils.deleteDir(file);
        } else {
            file.delete();
        }
    }

    @DeleteMapping()
    public ResultVO moduleRemove (@RequestBody ModuleBean param) {
        if (param.layui) {
            removeJavaFiles("/resources/public/");
        }
        if (param.bmob) {
            removeVueFiles("/common/bmob.js");
        }
        if (param.codeOnline) {
            removeVueFiles("/page/module/code-online/");
        }
        if (param.advanceComponents) {
            removeVueFiles("/components/advance/");
            removeVueFiles("/dex-advance.js");
        }
        if (param.mobile) {
            removeVueFiles("/page/mobile/");
        }

        if (param.javaTemplateRemove) {
            String[] arr = new String[]{"/code-gen", "/zookeeper-inspect", "/kafka-inspect",
                    "/table-gen", "/wanglei", "/table-gen", "/module-clear",
                    "/menu", "/org"};
            for (String route : arr) {
                sysMenuService.deleteByRoute(route);
            }

            removeJavaFiles("/java/com/oscar/binder/");
            removeJavaFiles("/resources/mapper/binder/");
            removeJavaFiles("/resources/templates/");
            removeJavaTextFiles("/com/oscar/binder/gen/generater/");
        }
        if (param.vueTemplateRemove) {
            removeVueFiles("/page/gen");
            removeVueFiles("/static", "/gen/");
        }
        if (param.wechatExamples) {
            removeWechatFiles("/examples");
        }

        if (param.banner) {
            deleteTable("tb_banner");
            removeJavaFiles("/java/com/oscar/module/banner");
            removeVueFiles("/page/module/banner");
            sysMenuService.deleteByRoute("/tb_banner");
        }

        if (param.order) {
            deleteTable("tb_order");
            removeJavaFiles("/java/com/oscar/module/order");
            removeVueFiles("/page/module/order");
            sysMenuService.deleteByRoute("/tb_order");
        }
        if (param.product) {
            deleteTable("tb_product");
            deleteTable("tb_product_category");
            removeJavaFiles("/java/com/oscar/module/product");
            removeVueFiles("/page/module/product");
            sysMenuService.deleteByRoute("/tb_product");
            sysMenuService.deleteByRoute("/tb_product_category");
        }

        if (param.location) {
            deleteTable("tb_location");
            removeJavaFiles("/java/com/oscar/module/location");
            removeVueFiles("/page/module/location");
            sysMenuService.deleteByRoute("/tb_location");
        }

        if (param.clockIn) {
            deleteTable("tb_clock_in");
            removeJavaFiles("/java/com/oscar/module/sign");
            removeVueFiles("/page/module/sign");
            sysMenuService.deleteByRoute("/tb_clock_in");
        }
        if (param.leave) {
            deleteTable("tb_leave");
            removeJavaFiles("/java/com/oscar/module/leave");
            removeVueFiles("/page/module/leave");
            sysMenuService.deleteByRoute("/tb_leave");
        }
        if (param.course) {
            deleteTable("tb_course");
            removeJavaFiles("/java/com/oscar/module/course");
            removeVueFiles("/page/module/course");
            sysMenuService.deleteByRoute("/tb_course");
        }
        if (param.wenjuan) {
            deleteTable("tb_wenjuan");
            deleteTable("tb_wenjuan_record");
            removeJavaFiles("/java/com/oscar/module/wenjuan");
            removeVueFiles("/page/module/wenjuan");
            sysMenuService.deleteByRoute("/tb_wenjuan");
            sysMenuService.deleteByRoute("/tb_wenjuan_edit");
            sysMenuService.deleteByRoute("/tb_wenjuan_record");
        }

        if (param.notice) {
            deleteTable("tb_cms_notice");
            removeJavaFiles("/java/com/oscar/module/cms/notice");
            removeVueFiles("/page/module/cms/notice");
            sysMenuService.deleteByRoute("/tb_cms_notice");
        }
        if (param.cmsInfo) {
            deleteTable("tb_cms_info");
            removeJavaFiles("/java/com/oscar/module/cms/info");
            removeVueFiles("/page/module/cms/info");
            sysMenuService.deleteByRoute("/tb_cms_info");
        }
        if (param.favorite) {
            deleteTable("tb_cms_favorite");
            removeJavaFiles("/java/com/oscar/module/cms/favorite");
            removeVueFiles("/page/module/cms/favorite");
            sysMenuService.deleteByRoute("/tb_cms_favorite");
        }
        if (param.reply) {
            deleteTable("tb_cms_reply");
            removeJavaFiles("/java/com/oscar/module/cms/reply");
            removeVueFiles("/page/module/cms/reply");
            sysMenuService.deleteByRoute("/tb_cms_reply");
        }

        deleteTable("tb_oscar_info");
        removeJavaFiles("/java/com/oscar/module/oscar");
        removeVueFiles("/page/module/oscar");
        sysMenuService.deleteByRoute("/tb_oscar_info");

        return ResultVO.success();
    }

    private void deleteTable (String tableName) {
        System.out.println("开始删除表：" + tableName);
        systemMapper.exec("DROP TABLE IF EXISTS " + tableName);
    }

    private String javaRoot () {
        return configSysProperties.getProjectRootDir() + File.separator + applicationName + "/java";
    }

    private String getVueRoot () {
        return configSysProperties.getProjectRootDir() + File.separator + applicationName + "/web";
    }

    private String getWechatRoot () {
        return configSysProperties.getProjectRootDir() + File.separator + applicationName + "/wechat";
    }
}
