package com.oscar.gen.android;

import cn.hutool.extra.template.TemplateException;
import com.google.common.io.Files;
import com.oscar.gen.CodeGen;
import com.oscar.gen.FreeMarkerUtils;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.List;

/**
 * Desc :
 *
 * @author wl
 * Date:    2019/4/1 14:27
 * @version 1.0
 */
public class AndroidGen {
    public static void gen (CodeGen codeGen) throws IOException, TemplateException, freemarker.template.TemplateException {
        if(!codeGen.getOption().android){
            return;
        }

        String src = codeGen.getAndroid().getFilePath();
        String module = codeGen.getModuleName();
        codeGen.android.setPackageName("com.oscar.modules."+module);
        String javaPath = src + "java/com/oscar/modules/"+module+"/";
        String resPath = src + "res/layout/"+module+"/layout/";
        FreeMarkerUtils.gen(javaPath, codeGen.getClassName() + "Activity.java", codeGen, "android/activity.ftl", "");
        FreeMarkerUtils.gen(javaPath, codeGen.getClassName() + "FormActivity.java", codeGen, "android/formActivity.ftl", "");
        FreeMarkerUtils.gen(javaPath, codeGen.getClassName() + "Adapter.java", codeGen, "android/adapter.ftl", "");
        FreeMarkerUtils.gen(javaPath, codeGen.getClassName() + ".java", codeGen, "android/entity.ftl", "");

        FreeMarkerUtils.gen(resPath, "activity_" + codeGen.getTableName() + ".xml", codeGen, "android/activity_xml.ftl", "");
        FreeMarkerUtils.gen(resPath, "cardview_" + codeGen.getTableName() + ".xml", codeGen, "android/cardview_layout.ftl", "");
        FreeMarkerUtils.gen(resPath, "activity_form_" + codeGen.getTableName() + ".xml", codeGen, "android/activity_form_xml.ftl", "");

        updateAndroidManifest(codeGen);
    }

    // 更新AndroidManifest.xml 添加Activity
    private static void updateAndroidManifest(CodeGen codeGen) throws IOException {
        String xml = codeGen.getRoot()+"/android/app/src/main/AndroidManifest.xml";
        final List<String> list = Files.readLines(new File(xml), Charset.forName("utf-8"));

        String androidName = codeGen.android.getPackageName()+"."+ codeGen.getClassName();
        for (String line : list) {
            if(line.contains(androidName+"FormActivity") || line.contains(androidName+"Activity")){
                return;
            }
        }

        StringBuilder out = new StringBuilder();
        for (String line : list) {
            if(line.trim().startsWith("</application>")){
                out.append("        <activity").append("\n")
                        .append("            android:name=\".modules."+ codeGen.getModuleName()+"."+ codeGen.getClassName()+"Activity\"").append("\n")
                        .append("            android:label=\""+ codeGen.android.getTitle()+"\" />").append("\n");
                out.append("        <activity").append("\n")
                        .append("            android:name=\".modules."+ codeGen.getModuleName()+"."+ codeGen.getClassName()+"FormActivity\"").append("\n")
                        .append("            android:label=\""+ codeGen.android.getTitle()+"表单\" />").append("\n");

            }
            out.append(line).append("\n");
        }

        Files.write(out.toString().getBytes(), new File(xml));
    }
}
