package com.oscar.gen.express;

import cn.hutool.extra.template.TemplateException;
import com.google.common.io.Files;
import com.oscar.gen.CodeGen;
import com.oscar.gen.FreeMarkerUtils;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.List;

public class ExpressGen {
    public static void gen (CodeGen codeGen) throws IOException, TemplateException, freemarker.template.TemplateException {
        if (!codeGen.getOption().express) {
            return;
        }
        String fileName = codeGen.getClassName();

        //        FreeMarkerUtils.gen(src, fileName, codeGen, "ssh/controller.ftl", "Controller.java");
        FreeMarkerUtils.gen(codeGen.getExpress().getSqlMapperPath(), fileName, codeGen, "express/mapper.ftl", "Mapper.js");
        appendRoute(codeGen);
    }

    /**
     * 生成路由 routes/oscar.js
     */
    private static void appendRoute (CodeGen codeGen) throws IOException {
        String uri = codeGen.getRequestUri().replaceFirst("/tb", "");
        String serviceName = codeGen.getClassName() + "Service"; //TbCmsNoticeService
        String mapperName = codeGen.getClassName() + "Mapper"; //TbCmsNoticeMapper

        File file = new File(codeGen.getExpress().getRoutePath() + "oscar.js");
        final List<String> list = Files.readLines(file, Charset.forName("utf-8"));
        StringBuilder out = new StringBuilder();

        String mapperCode = "const " + serviceName + " = require('../server/mysql/mapper/tb/" + mapperName + "')";
        for (String line : list) {
            if (line.equals(mapperCode)) {  //已经生成过，就不重复生成了
                return;
            }
        }

        StringBuilder routeCode = new StringBuilder();
        routeCode.append("router.get('").append(uri).append("/page', async(req, res) => {").append("\n")
                .append("    let list = await ").append(serviceName).append(".pageList(req.query)").append("\n")
                .append("    res.send(list)").append("\n")
                .append("})").append("\n");

        routeCode.append("router.get('").append(uri).append("/list', async(req, res) => {").append("\n")
                .append("    let list = await ").append(serviceName).append(".findList(req.query)").append("\n")
                .append("    res.send(util.success(list))").append("\n")
                .append("})").append("\n");

        routeCode.append("router.post('").append(uri).append("', async(req, res) => {").append("\n")
                .append("    await ").append(serviceName).append(".save(req.body)").append("\n")
                .append("    res.send(util.success())").append("\n")
                .append("})").append("\n");

        routeCode.append("router.delete('").append(uri).append("', async(req, res) => {").append("\n")
                .append("    await ").append(serviceName).append(".delete(req.body)").append("\n")
                .append("    res.send(util.success())").append("\n")
                .append("})").append("\n");

        for (String line : list) {
            out.append(line).append("\n");
            if (line.trim().startsWith("// mapper code start")) {
                out.append(mapperCode).append("\n");
            }
            if (line.trim().startsWith("// route code start")) {
                out.append(routeCode).append("\n");
            }
        }

        Files.write(out.toString().getBytes(), file);
    }
}
