package com.oscar.gen.vux;

import cn.hutool.extra.template.TemplateException;
import com.google.common.io.Files;
import com.oscar.gen.CodeGen;
import com.oscar.gen.FreeMarkerUtils;

import java.io.File;
import java.io.IOException;

public class VuxGen {
    public static void gen (CodeGen codeGen) throws IOException, TemplateException, freemarker.template.TemplateException {
        if(!codeGen.getOption().vux){
            return;
        }

        final Vux vux = codeGen.getVux();

        //生成m_desktop.vue
        if (vux.isMenuUpdate()) {
            String mobilePath = codeGen.getRoot() + "/web/src/page/mobile/";
            FreeMarkerUtils.gen(mobilePath, "m_desktop", codeGen, "vux/vux-m_desktop.ftl", ".vue");
        }

        //更新style
        if(codeGen.styleUpdate){
            String stylePath = codeGen.getRoot()+"/web/src/style/";
            String mobilePath = codeGen.getRoot()+"/web/src/page/mobile/";
            FreeMarkerUtils.gen(stylePath, "style.less", codeGen, "vux/style.ftl", "");
            FreeMarkerUtils.gen(mobilePath, "m_login.vue", codeGen, "vux/vux-m_login.ftl", "");
        }

        //生成业务代码
        if(codeGen.getOption().vux){
            String tableName = vux.getFileName(); //tb_oscar_info
            String list = tableName + "_list"; //tb_oscar_info_list
            String detail = tableName + "_detail";
            FreeMarkerUtils.gen(vux.getFilePath(), list, codeGen, "vux/vux-list.ftl", ".vue");
            FreeMarkerUtils.gen(vux.getFilePath(), detail, codeGen, "vux/vux-detail.ftl", ".vue");

            //修改登录页logo
            String logoPath = codeGen.getRoot() + "/web/static/style/mobile/logo/";
            Files.copy(new File(logoPath + vux.getLogo() + ".png"), new File(logoPath + "0.png"));
            refreshMobileRouter(codeGen, detail);
            refreshMobileRouter(codeGen, list);
        }

    }

    private static void refreshMobileRouter (CodeGen codeGen, String vueComponent) {
        String routerJs = codeGen.getProjectPath().replaceAll("/java/", "/web/src/router/index.js");
        try {
            FreeMarkerUtils.refreshMobileRouter(vueComponent, codeGen.getModuleName(), vueComponent, routerJs);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


}
