package com.oscar.gen.wechat;

import cn.hutool.extra.template.TemplateException;
import com.google.common.io.Files;
import com.oscar.gen.CodeGen;
import com.oscar.gen.FreeMarkerUtils;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.List;

public class WechatGen {
    public static void gen (CodeGen codeGen) throws IOException, TemplateException, freemarker.template.TemplateException {
        if(!codeGen.getOption().wechat){
            return;
        }
        final Wechat wechat = codeGen.getWechat();

        String output = codeGen.getWechat().getFilePath();
        if(wechat.isGenForm()){
            FreeMarkerUtils.gen(output, "form.wxml", codeGen, "wechat/form/wechat-form-wxml.ftl", "");
            FreeMarkerUtils.gen(output, "form.js", codeGen, "wechat/form/wechat-form-js.ftl", "");
            FreeMarkerUtils.gen(output, "form.json", codeGen, "wechat/form/wechat-form-json.ftl", "");
            FreeMarkerUtils.gen(output, "form.wxss", codeGen, "wechat/form/wechat-form-wxss.ftl", "");
        }

        FreeMarkerUtils.gen(output, "index.wxml", codeGen, "wechat/index/wechat-index-wxml.ftl", "");
        FreeMarkerUtils.gen(output, "index.js", codeGen, "wechat/index/wechat-index-js.ftl", "");
        FreeMarkerUtils.gen(output, "index.json", codeGen, "wechat/index/wechat-index-json.ftl", "");
        FreeMarkerUtils.gen(output, "index.wxss", codeGen, "wechat/index/wechat-index-wxss.ftl", "");

        FreeMarkerUtils.gen(output, "detail.wxml", codeGen, "wechat/detail/wechat-detail-wxml.ftl", "");
        FreeMarkerUtils.gen(output, "detail.js", codeGen, "wechat/detail/wechat-detail-js.ftl", "");
        FreeMarkerUtils.gen(output, "detail.json", codeGen, "wechat/detail/wechat-detail-json.ftl", "");
        FreeMarkerUtils.gen(output, "detail.wxss", codeGen, "wechat/detail/wechat-detail-wxss.ftl", "");

        updateAppJson(codeGen);
    }

    // 更新微信小程序的app.json
    private static void updateAppJson(CodeGen codeGen) throws IOException {
        String appjson = codeGen.getRoot()+"/wechat/app.json";
        final List<String> list = Files.readLines(new File(appjson), Charset.forName("utf-8"));

        for (String line : list) {
            if(line.trim().startsWith("\"pages/"+ codeGen.getWechat().getFileName())){
                return;
            }
        }

        StringBuilder out = new StringBuilder();
        for (String line : list) {
            out.append(line).append("\n");
            if(line.startsWith("  \"pages\": [")){
                out.append("    \"pages/").append(codeGen.getWechat().getFileName()).append("/index\",").append("\n");

                if(codeGen.getWechat().isGenForm()){
                    out.append("    \"pages/").append(codeGen.getWechat().getFileName()).append("/form\",").append("\n");
                }
            }
        }

        Files.write(out.toString().getBytes(), new File(appjson));
    }
}
