package com.oscar.module.cms.info;

import com.github.pagehelper.PageInfo;
import com.oscar.base.AbstractController;
import com.oscar.base.enums.CssStyleTypeEnum;
import com.oscar.base.vo.ResultVO;
import com.oscar.module.cms.reply.TbCmsReply;
import com.oscar.module.cms.reply.TbCmsReplyService;
import com.oscar.module.favorite.TbCmsFavorite;
import com.oscar.module.favorite.TbCmsFavoriteService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@RestController
@RequestMapping("/tb/cms/info")
@AllArgsConstructor
public class TbCmsInfoController extends AbstractController<TbCmsInfoService, TbCmsInfo>{
    final TbCmsReplyService tbCmsReplyService;
    final TbCmsFavoriteService tbCmsFavoriteService;

    @GetMapping("advance/page")
    public PageInfo<TbCmsInfo> advancePageList (HttpServletRequest request, TbCmsInfo tbCmsInfo) {
        PageInfo<TbCmsInfo> pageInfo = service.pageList(request, tbCmsInfo);

        for (TbCmsInfo x : pageInfo.getList()) {
            if("未审核".equals(x.getState())){
                x.setType(CssStyleTypeEnum.warning.getText());
            }else if("审核通过".equals(x.getState())){
                x.setDisabled(true);
                x.setType(CssStyleTypeEnum.success.getText());
            }else if("驳回".equals(x.getState())){
                x.setDisabled(true);
                x.setType(CssStyleTypeEnum.danger.getText());
            }
        }
        return pageInfo;
    }

    @Override
    public PageInfo<TbCmsInfo> pageList (HttpServletRequest request, TbCmsInfo tbCmsInfo) {
        final PageInfo<TbCmsInfo> tbCmsInfoPageInfo = service.pageList(request, tbCmsInfo);
        queryReplyAndFavorite(tbCmsInfoPageInfo.getList());
        return tbCmsInfoPageInfo;
    }

    private void queryReplyAndFavorite (List<TbCmsInfo> all){
        for(TbCmsInfo info : all){
            TbCmsReply param = new TbCmsReply();
            param.setCmsId(info.getId());
            final List<TbCmsReply> replyList = tbCmsReplyService.findList(param);
            info.setReplyList(replyList); //留言列表

            TbCmsFavorite q1 = new TbCmsFavorite();
            q1.setCmsId(info.getId());
            q1.setType("收藏");
            final int favoriteCount = tbCmsFavoriteService.count(q1);
            info.setFavorite(favoriteCount);
            q1.setType("点赞");
            final int praiseCount = tbCmsFavoriteService.count(q1);
            info.setPraise(praiseCount);
        }
    }

    @Override
    public ResultVO findList (TbCmsInfo tbCmsInfo) {
        Integer userId = tbCmsInfo.getUserId();
        tbCmsInfo.setUserId(null); //查询所有人发的帖子
        final List<TbCmsInfo> all = service.findList(tbCmsInfo);
        queryReplyAndFavorite(all);

        //查询我的收藏
        TbCmsFavorite query = new TbCmsFavorite();
        query.setUserId(userId);
        query.setSource("tb_cms_info");
        final List<TbCmsFavorite> myFavoriteList = tbCmsFavoriteService.findList(query);
         Set<Integer> favoriteIds = new HashSet<>(); //收藏的id
        Set<Integer> praiseIds = new HashSet<>(); //点赞的id
        myFavoriteList.forEach(x->{
            if("点赞".equals(x.getType())){
                praiseIds.add(x.getCmsId());
            }
            if("收藏".equals(x.getType())){
                favoriteIds.add(x.getCmsId());
            }
        });

        all.forEach(x->{
            if(favoriteIds.contains(x.getId())){
                x.setInMyFavorite(true);
            }
            if(praiseIds.contains(x.getId())){
                x.setHasPraised(true);
            }
        });

        return success(all);
    }

    //转发
    @PostMapping("forward")
    public ResultVO forward (@RequestBody TbCmsInfo tbCmsInfo) {
        service.get(tbCmsInfo).ifPresent(x->{
            x.setForward(x.getForward()+1);
            service.save(x);
        });

        return success();
    }

}
