package com.oscar.module.location;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.github.pagehelper.PageInfo;
import com.oscar.base.AbstractController;
import com.oscar.base.enums.CssStyleTypeEnum;
import com.oscar.base.vo.ResultVO;
import com.oscar.module.course.TbCourse;
import com.oscar.module.course.TbCourseService;
import com.oscar.module.leave.TbLeave;
import com.oscar.module.leave.TbLeaveService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

//定位签到
@RestController
@RequestMapping("/tb/location")
public class TbLocationController extends AbstractController<TbLocationService, TbLocation> {
    @Autowired
    TbLeaveService tbLeaveService;
    @Autowired
    TbCourseService tbCourseService;

    @GetMapping("advance/page")
    public PageInfo<TbLocation> advancePageList (HttpServletRequest request, TbLocation tbLocation) {
        PageInfo<TbLocation> pageInfo = service.pageList(request, tbLocation);

        for (TbLocation x : pageInfo.getList()) {
            if("未审核".equals(x.getState())){
                x.setType(CssStyleTypeEnum.warning.getText());
            }else if("审核通过".equals(x.getState())){
                x.setDisabled(true);
                x.setType(CssStyleTypeEnum.success.getText());
            }else if("驳回".equals(x.getState())){
                x.setDisabled(true);
                x.setType(CssStyleTypeEnum.danger.getText());
            }
        }
        return pageInfo;
    }

    class StatEntity {
        public int lateNum = 0; // 迟到
        public int leaveEarlyNum = 0;//早退
        public int dutyOffNum = 0;//旷工
        public int leaveNum = 0;//请假
    }

    /**
     * 每月统计迟到 早退 旷工 请假次数
     */
    @GetMapping("month/stat")
    public ResultVO statByMonty (TbLocation tbLocation) {
        final Map<String, StatEntity> map = new HashMap<>();
        service.findList(tbLocation).forEach(data -> {
            StatEntity entity = map.get(data.getCnName());
            if (entity == null)
                entity = new StatEntity();

            map.put(data.getCnName(), entity);
            if ("迟到".equals(data.getState())) {
                entity.lateNum += 1;
            } else if ("早退".equals(data.getState())) {
                entity.leaveEarlyNum += 1;
            } else if ("旷工".equals(data.getState())) {
                entity.dutyOffNum += 1;
            }
        });

        TbLeave query = new TbLeave();
        query.setMonth(tbLocation.getMonth());
        tbLeaveService.findList(query).forEach(data -> {
            StatEntity entity = map.get(data.getCnName());
            if (entity == null)
                entity = new StatEntity();

            map.put(data.getCnName(), entity);
            entity.leaveNum += 1;
        });

        return success(map);
    }

    @Override
    public ResultVO save (TbLocation tbLocation) {
        //坐标计算与目标位置的距离
        tbLocation.setInTargetScope("是");
        tbLocation.setState("正常");

        final DateTime now = DateUtil.date();
        final int hour = now.hour(true);
        final int minute = now.minute();
        boolean late = hour>8 || (hour ==8 && minute>0); //大于8:00
        if ("on".equalsIgnoreCase(tbLocation.getDutyOnOff()) && late) {
            tbLocation.setState("迟到");
        }

        boolean leaveEarly = hour < 18 || (hour==18 && minute<0); //小于18:00
        if ("off".equalsIgnoreCase(tbLocation.getDutyOnOff()) && leaveEarly) {
            tbLocation.setState("早退");
        }

        service.save(tbLocation);
        return success();
    }

    /**
     * 有课程签到码的时候执行
     */
    @PostMapping("courseSign")
    public ResultVO courseSign (@RequestBody TbLocation tbLocation) {
        //查询课程
        TbCourse courseQuery = new TbCourse();
        courseQuery.setId(tbLocation.getCourseId());
        TbCourse course = tbCourseService.get(courseQuery).orElse(null);
        if (course == null) {
            return error("课程号不正确");
        }
        if (!course.getSignCode().equalsIgnoreCase(tbLocation.getSignCode())) {
            return error("签到码不正确");
        }

        final Double targetLon = course.getLon();
        final Double targetLat = course.getLat();
        final String startTimeLimit = course.getStartTimeLimit();
        final String endTimeLimit = course.getEndTimeLimit();

        //坐标计算与目标位置的距离
        if (service.getDistance(targetLat, targetLon, tbLocation.getLat(), tbLocation.getLon()) > course.getScopeLimit()) {
            return error("当前签到地点不在教师范围内");
        }

        String[] startArr = startTimeLimit.split(":");
        String[] endArr = endTimeLimit.split(":");
        int startHour = Integer.parseInt(startArr[0]);
        int endHour = Integer.parseInt(endArr[0]);
        int startMinute = Integer.parseInt(startArr[1]);
        int endMinute = Integer.parseInt(endArr[1]);

        final DateTime now = DateUtil.date();
        final int hour = now.hour(true);
        final int minute = now.minute();
        boolean lessThanStart = hour<startHour || (hour ==startHour && minute<startMinute);
        boolean greatThanEnd = hour>endHour || (hour ==endHour && minute>endMinute);
        if (lessThanStart || greatThanEnd) {
            return error("当前签到时间不在正确范围内");
        }

        tbLocation.setInTargetScope("是");
        tbLocation.setState("正常");
        service.save(tbLocation);
        return success();
    }

}
