package com.oscar.sys;

import com.oscar.sys.user.SysUser;
import com.oscar.sys.user.SysUserService;
import org.apache.ibatis.io.Resources;
import org.apache.ibatis.jdbc.ScriptRunner;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.sql.DataSource;
import java.io.Reader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Order(-100)
public class GlobalData {
    private final ApplicationContext applicationContext;
    private final SysUserService sysUserService;

    @Value("${spring.application.name}")
    private String appName;
    @Value("${spring.datasource.url}")
    private String url;
    @Value("${spring.datasource.username}")
    private String username;
    @Value("${spring.datasource.password}")
    private String password;
    @Value("${spring.datasource.driver-class-name}")
    private String driverClassName;

    private static Map<Integer, SysUser> userMap;

    @Autowired
    public GlobalData (ApplicationContext applicationContext, SysUserService sysUserService) {
        this.applicationContext = applicationContext;
        this.sysUserService = sysUserService;
    }

    private void createDb () {
        final String[] split = url.split(appName);
        String dbSql = "CREATE DATABASE IF NOT EXISTS " + appName + " DEFAULT CHARACTER SET utf8 DEFAULT COLLATE utf8_general_ci";
        try {
            Class.forName(driverClassName);
            Connection conn = DriverManager.getConnection(split[0] + "information_schema", username, password);
            Statement statement = conn.createStatement();
            statement.execute(dbSql);

            dbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void dbInit () throws Exception{
        DataSource dataSource = applicationContext.getBean(DataSource.class);
        boolean firstRun = true;
        try (Connection connection = dataSource.getConnection(); Statement statement = connection.createStatement()) {
            ResultSet rs = statement.executeQuery("SELECT COUNT(TABLE_NAME) AS CNT FROM information_schema.TABLES WHERE TABLE_SCHEMA='" + appName + "'");
            while (rs.next()) {
                int cnt = rs.getInt("CNT");
                if (cnt > 0) {
                    firstRun = false;
                }
            }

            if (firstRun) {
                ScriptRunner runner = new ScriptRunner(connection);
                Reader resourceAsReader = Resources.getResourceAsReader("data.sql");
                runner.runScript(resourceAsReader);
            }
        }
    }

    @PostConstruct
    public void init () {
        createDb();
        refreshUser();
    }

    public void refreshUser(){
        userMap = new HashMap<>();
        final List<SysUser> list = sysUserService.findUserRoleList(new SysUser());
        for (SysUser user : list) {
            userMap.put(user.getId(), user);
        }
    }

    public static SysUser getSysUser (Integer userId) {
        return userMap.get(userId);
    }

    /**
     * 登录用户姓名
     * @param userId
     */
    public static String getUserName (Integer userId) {
        final SysUser sysUser = getSysUser(userId);
        if(sysUser == null)
            return "";
        else
            return sysUser.getCnName();
    }
}
