package com.oscar.sys;

import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;
import com.oscar.base.config.ConfigSysProperties;
import com.oscar.sys.user.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.DigestUtils;
import org.springframework.util.StringUtils;

import javax.annotation.PostConstruct;
import java.util.concurrent.TimeUnit;

@Component
public class SessionProvider {
    @Autowired
    ConfigSysProperties config;
    static Cache<Object, Object> cb = null;

    @PostConstruct
    public void init () {
        cb = CacheBuilder.newBuilder().maximumSize(30000)
                .expireAfterAccess(config.getSessionTimeout(), TimeUnit.MINUTES)
                .build();
    }

    public void encrypt(SysUser sysUser){
        sysUser.setPassword(encrypt(sysUser.getPassword()));
    }

    public String encrypt (String plainText) {
        if (StringUtils.isEmpty(plainText)) return "";
        return DigestUtils.md5DigestAsHex((plainText + config.getSalt()).getBytes());
    }

    public void create (Object key, Object value) {
        cb.put(key, value);
    }

    public void invalidate (Object key) {
        if (key == null) return;
        cb.invalidate(key);
    }

    public final boolean isValid (Object key) {
        if (key == null) return false;

        Object ifPresent = cb.getIfPresent(key);
        if (ifPresent == null)
            return false;

        return true;
    }
}
