package com.oscar.sys.common;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.oscar.base.AbstractController;
import com.oscar.base.config.ConfigSysProperties;
import com.oscar.base.util.FileUtils;
import com.oscar.base.vo.ResultVO;
import com.oscar.sys.SystemMapper;
import com.oscar.sys.TreeFactory;
import com.oscar.sys.menu.SysMenu;
import com.oscar.sys.menu.SysMenuService;
import com.oscar.sys.user.SysUser;
import com.oscar.sys.user.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.method.annotation.StreamingResponseBody;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

//@RestController
//@RequestMapping("/sys")
public class CommonController extends AbstractController {
    @Autowired
    ConfigSysProperties config;
    @Autowired
    SystemMapper systemMapper;
    @Autowired
    SysUserService sysUserService;
    @Autowired
    SysMenuService sysMenuService;

    /**
     * 通过1569544734617-8.jpg 查询image base64编码
     */
    @GetMapping("images/base64")
    public ResultVO getBase64ImagesByName(@RequestParam(value = "list") List<String> list){
        List<String> base64List = new ArrayList<>();
        try {
            for (String imgUrl : list) {
                final String base64 = FileUtils.imageToBase64(config.getUploadPath(), imgUrl);
                base64List.add(base64);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }finally {
            return success(base64List);
        }
    }
    @GetMapping("image/base64")
    public ResultVO getBase64ImageByName(String imgUrl){
        String base64 = "";
        try {
            base64 = FileUtils.imageToBase64(config.getUploadPath(), imgUrl);
        } catch (Exception e) {
            e.printStackTrace();
        }finally {
            return success(base64);
        }
    }

    @PostMapping("/upload")
    public ResultVO upload (@RequestParam("file") MultipartFile myfile, @RequestParam(value = "category", required = false) String category) {
        String fileName = "";
        if (myfile != null && !myfile.isEmpty()) {
//            String contentType = myfile.getContentType();
//            BufferedImage image = ImageIO.read(myfile.getInputStream());
            fileName = System.currentTimeMillis()+FileUtils.getSuffix(myfile.getOriginalFilename());

            try {
                if(StringUtils.isEmpty(category) || "undefined".equalsIgnoreCase(category)){
                    category = "default";
                }
                String dirPath = config.getUploadPath() + category + "/" + DateUtil.today()+"/";
                File dir = new File(dirPath);
                if(!dir.exists()){
                    dir.mkdirs();
                }

                myfile.transferTo(new File( dirPath + fileName));
            } catch (IOException e) {
                e.printStackTrace();
                fileName = "";
            }
        }
        return ResultVO.success().set(category + "/" + DateUtil.today()+"/"+fileName);
    }

    @Async
    @GetMapping("/download")
    public StreamingResponseBody upload (String fileName, HttpServletResponse response) throws UnsupportedEncodingException {
        if(StringUtils.isEmpty(fileName)){
            System.err.println("fileName is null");
            return null;
        }

        File file = new File(config.getUploadPath() + fileName);
        if (file != null && file.exists()) {
            super.setResponseAsAttachment(response,fileName);

            try {
                final InputStream is = new FileInputStream(file);
                return outputStream -> {
                    int nRead;
                    byte[] data = new byte[1024];
                    while ((nRead = is.read(data, 0, data.length)) != -1) {
                        outputStream.write(data, 0, nRead);
                    }
                    outputStream.flush();
                    outputStream.close();
                };
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    @GetMapping("/tree")
    public ResultVO tree (String t) {
        List<Map<String, Object>> list = systemMapper.findList("select * from " + t);
        List<Map<String, Object>> mapStream = list.stream().map(m -> {
            Map<String, Object> nm = new HashMap<>();
            for (String key : m.keySet()) {
                nm.put(StrUtil.toCamelCase(key), m.get(key));
            }
            return nm;
        }).collect(Collectors.toList());
        return ResultVO.success().setList(TreeFactory.buildLM(mapStream));
    }

    @RequestMapping("/static/page")
    public String login(){
        return "/pages/login.html";
    }

    @GetMapping("main/stat")
    public ResultVO stat(){
        Map<String,Integer> map = new HashMap<>();
        map.put("user_count",sysUserService.count(new SysUser()));
        map.put("module_count",sysMenuService.count(new SysMenu()));
        map.put("view_count",895);
        return ResultVO.success().set(map);
    }

}
