package com.oscar.sys.common;

import cn.hutool.json.JSON;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.oscar.base.AbstractController;
import com.oscar.base.config.ConfigSysProperties;
import com.oscar.base.util.EncryptUtils;
import com.oscar.base.util.SmsUtils;
import com.oscar.base.vo.ResultVO;
import com.oscar.sys.RequestFilter;
import com.oscar.sys.SessionProvider;
import com.oscar.sys.role.SysRole;
import com.oscar.sys.role.SysRoleService;
import com.oscar.sys.user.SysUser;
import com.oscar.sys.user.SysUserService;
import com.oscar.sys.userrole.SysUserRole;
import com.oscar.sys.userrole.SysUserRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

//@RestController
public class LoginController extends AbstractController {
    @Autowired
    ConfigSysProperties config;
    @Autowired
    SysUserService sysUserService;
    @Autowired
    SysUserRoleService sysUserRoleService;
    @Autowired
    SysRoleService sysRoleService;
    @Autowired
    SessionProvider sessionProvider;
    @Autowired
    RestTemplate restTemplate;

    @GetMapping("/version")
    public ResultVO version () {
        Map<String, String> version = new HashMap<>();
        version.put("version", "java");
        return success(version);
    }

    @GetMapping("/code2session")
    public ResultVO code2session (String appid, String secret, String js_code) {
        Map<String, String> param = new HashMap<>();
        param.put("appid", appid);
        param.put("secret", secret);
        param.put("js_code", js_code);
        String body = restTemplate.getForObject("https://api.weixin.qq.com/sns/jscode2session", String.class, param);

        JSONObject jsonObject = JSONUtil.parseObj(body);
        Map<String,String> session = new HashMap<>();
        session.put("session_key",jsonObject.getStr("session_key"));
        session.put("openid",jsonObject.getStr("openid"));

        Map<String,Map<String,String>> result = new HashMap<>();
        result.put("data",session);
        return success(result);
    }

    @GetMapping("/timeout")
    public ResultVO timeout () {
        return ResultVO.timeout("Session Timeout!");
    }

    @GetMapping("/logout")
    public ResultVO logout (HttpServletRequest request) {
        sessionProvider.invalidate(RequestFilter.getToken(request));
        return success();
    }

    /**
     * change pwd
     * id or username needed
     * password and newPassword needed
     *
     * @return
     */
    @PostMapping("/password")
    public ResultVO changePwd (@RequestBody SysUser user) {
        if (StringUtils.isEmpty(user.getNewPassword())) {
            return warn("新密码不能设置为空");
        }

        sessionProvider.encrypt(user);
        List<SysUser> list = sysUserService.findList(user);
        if (list.isEmpty()) {
            return warn("用户名或密码不正确");
        } else if (list.size() == 1) {
            final SysUser first = list.get(0);
            first.setPassword(sessionProvider.encrypt(user.getNewPassword()));
            sysUserService.save(first);
            return success();
        } else {
            return warn("More than one user found.");
        }
    }

    @PostMapping("sms/code")
    public ResultVO sendSmsCode (@RequestBody SysUser sysUser) {
        String phone = sysUser.getMobile();
        int code = SmsUtils.randomCodeGenerate(phone);
        if (code == -1) {
            //重复发送
            return warn("过于频繁，请稍后发送");
        } else {
            SmsUtils.sendSmsCode(phone, String.valueOf(code));
            return success();
        }
    }

    @PostMapping(value = "/m/login", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResultVO mobileLogin (@RequestBody SysUser user) {
        if (StringUtils.isEmpty(user.getUsername()) || StringUtils.isEmpty(user.getPassword())) {
            return warn("用户名或密码不正确");
        }

        sessionProvider.encrypt(user);
        List<SysUser> list = sysUserService.findList(user);
        if (list.isEmpty()) {
            return warn("用户名或密码不正确");
        } else {
            final SysUser first = list.get(0);
            setRoleAndOrg(first);
            return success(first);
        }
    }

    @PostMapping(value = "/uni/login", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResultVO uniLogin (@RequestBody SysUser user) {
        if (user.getSmsCode() != null) {
            if (!SmsUtils.isValid(user.getMobile(), user.getSmsCode())) {
                return warn("验证码不正确");
            }
        } else {
            sessionProvider.encrypt(user);
        }

        List<SysUser> list = sysUserService.findList(user);
        if (list.isEmpty()) {
            return warn("账号或密码不正确");
        } else {
            final SysUser first = list.get(0);
            setRoleAndOrg(first);
            return success(first);
        }
    }

    /**
     * 设置role和org
     */
    private void setRoleAndOrg (SysUser user) {
        final SysUserRole param = new SysUserRole();
        param.setUserId(user.getId());
        final List<SysUserRole> roleList = sysUserRoleService.findList(param);
        final List<Integer> roleIds = roleList.stream().map(o -> {
            return o.getRoleId();
        }).collect(Collectors.toList());
        user.setRoleIds(roleIds);
        if (roleIds.size() > 0) {
            SysRole roleParam = new SysRole();
            roleParam.setId(roleIds.get(0));
            user.setRoleId(roleIds.get(0));
            sysRoleService.get(roleParam).ifPresent(role->{
                user.setRoleName(role.getName());
            });
        }
    }

    @PostMapping(value = "/login", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResultVO login (@RequestBody SysUser user) {
        sessionProvider.encrypt(user);

        List<SysUser> list = sysUserService.findList(user);
        if (list.isEmpty()) {
            return warn("用户名或密码不正确");
        } else {
            user = list.get(0);
            String token = EncryptUtils.encrypt(String.valueOf(user.getId()), config.getSalt());
            sessionProvider.create(token, user);

            //            user.setId(-1);
            setRoleAndOrg(user);
            Map<String, Object> m = new HashMap<>();
            m.put("token", new Token(token, config.getSessionTimeout()));
            m.put("user", user);

            return success().set(m);
        }
    }

    static class Token {
        String token;
        int timeout;

        public Token (String token, int timeout) {
            this.token = token;
            this.timeout = timeout;
        }

        public String getToken () {
            return token;
        }

        public int getTimeout () {
            return timeout;
        }
    }
}
