package com.oscar.sys.menu;

import com.github.pagehelper.PageInfo;
import com.oscar.base.AbstractController;
import com.oscar.base.enums.CssStyleTypeEnum;
import com.oscar.base.vo.ResultVO;
import com.oscar.sys.rolemenu.SysRoleMenu;
import com.oscar.sys.rolemenu.SysRoleMenuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/sys/menu")
public class SysMenuController extends AbstractController<SysMenuService, SysMenu> {
    @Autowired
    SysRoleMenuService sysRoleMenuService;

    @Override
    public PageInfo<SysMenu> pageList (HttpServletRequest request, SysMenu sysMenu) {
        List<SysMenu> treeList = new ArrayList<>();
        sysMenu.setEnabled(null);
        List<SysMenu> list = service.findList(sysMenu);

        list.forEach(menu -> {
            if(menu.getParentId() <= 0){
                menu.setHref(""); //一级目录的href，隐藏了
                if("on".equals(menu.getEnabled())){
                    menu.setType(CssStyleTypeEnum.success.getText());
                }else{
                    menu.setType(CssStyleTypeEnum.danger.getText());
                }
                treeList.add(menu);
                list.forEach(child ->{
                    if(menu.getId().equals(child.getParentId())){
                        if("on".equals(child.getEnabled())){
                            child.setType(CssStyleTypeEnum.success.getText());
                        }else{
                            child.setType(CssStyleTypeEnum.danger.getText());
                        }
                        menu.getChildren().add(child);
                    }
                });
            }
        });

        return new PageInfo<>(treeList);
    }

    // TODO 页面需要改url
    @GetMapping("menus")
    public ResultVO findMenus (HttpServletRequest request) {
        return success(service.findMenuList(request));
    }

    @GetMapping("children")
    public ResultVO findChildrenById (SysMenu sysMenu) {
        return success(service.findList(sysMenu));
    }

    @GetMapping("router")
    public ResultVO findRouterList (HttpServletRequest request) {
        return ResultVO.success().setList(service.findRouterList(request));
    }

    @PostMapping("insert")
    public ResultVO insert (@RequestBody SysMenu sysMenu) {
        //add 下级菜单
        service.save(sysMenu);
        //同时赋权
        SysRoleMenu roleMenu = new SysRoleMenu();
        roleMenu.setRoleId(1);
        roleMenu.setMenuId(sysMenu.getId());
        sysRoleMenuService.save(roleMenu);
        return success();
    }

    @Override
    public ResultVO delete (SysMenu sysMenu) {
        service.delete(sysMenu);
        //delete role menu
        SysRoleMenu roleMenu = new SysRoleMenu();
        roleMenu.setMenuId(sysMenu.getId());
        sysRoleMenuService.delete(roleMenu);
        return success();
    }
}
