package com.oscar.sys.menu;

import com.oscar.base.AbstractService;
import com.oscar.base.config.ConfigSysProperties;
import com.oscar.base.util.EncryptUtils;
import com.oscar.sys.RequestFilter;
import com.oscar.sys.TreeFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class SysMenuService extends AbstractService<SysMenuMapper, SysMenu> {
    @Autowired
    ConfigSysProperties config;

    public List<Router> findRouterList (HttpServletRequest request) {
        List<SysMenu> menuList = findMenuList(request);
        List<Router> routers = new ArrayList<>();
        routeBuild(new ArrayList<>(), menuList, routers);
        return routers;
    }

    public void routeBuild (List<String> meta, List<SysMenu> menuList, List<Router> routers) {
        for (SysMenu menu : menuList) {
            if (menu.getChildren() == null || menu.getChildren().isEmpty()) {
                List<String> current = new ArrayList<>(meta);// 原来是用Lists.newArrayList guava的
                current.add(menu.getName());

                routers.add(new Router(menu.getHref(), menu.getComponent(), current));
            } else {
                meta.add(menu.getName());
                routeBuild(meta, menu.getChildren(), routers);
            }
        }
        meta.clear();
    }

    public List<SysMenu> findMenuList (HttpServletRequest request) {
        String token = RequestFilter.getToken(request);
        String userIdStr = EncryptUtils.decrypt(token, config.getSalt());
        SysMenu param = new SysMenu();
        List<SysMenu> all = findList(param);
        List<SysMenu> list = new ArrayList<>();
        Map<Integer, SysMenu> idToMenuMap = new HashMap<>();

        all.forEach(sysMenu -> {
            idToMenuMap.put(sysMenu.getId(), sysMenu);
            if(sysMenu.getParentId() == -1){  //回到首页的menu
                list.add(sysMenu);
            }
        });

        param.setUserId(Integer.parseInt(userIdStr));
        List<SysMenu> childMenuList = dao.findChildMenuList(param);

        for (SysMenu menu : childMenuList) {
            ss(idToMenuMap, list, menu);
        }

        Set<Integer> duplicate = new HashSet<>();
        List<SysMenu> collect = list.stream().filter(m -> duplicate.add(m.getId())).collect(Collectors.toList());
        return TreeFactory.build(collect);
    }

    private void ss (Map<Integer, SysMenu> map, List<SysMenu> list, SysMenu menu) {
        if(menu == null) return;

        list.add(menu);

        if (menu.getParentId() == 0) {
            return;
        }

        SysMenu parent = map.get(menu.getParentId());
        ss(map, list, parent);
    }

    public void deleteByRoute(String route){
        SysMenu menu = new SysMenu();
        menu.setHref(route);
        delete(menu);
    }
}
