package com.oscar.sys.user;

import com.github.pagehelper.PageInfo;
import com.oscar.base.AbstractController;
import com.oscar.base.util.SmsUtils;
import com.oscar.base.vo.ResultVO;
import com.oscar.sys.SessionProvider;
import com.oscar.sys.userrole.SysUserRole;
import com.oscar.sys.userrole.SysUserRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/sys/user")
public class SysUserController extends AbstractController<SysUserService, SysUser> {
    @Autowired
    SysUserRoleService sysUserRoleService;
    @Autowired
    SysUserExtService sysUserExtService;
    @Autowired
    SessionProvider sessionProvider;

    @GetMapping("all")
    public PageInfo<SysUser> findUserRoleList (HttpServletRequest request, SysUser sysUser) {
        final PageInfo<SysUser> pageInfo = service.pageList(request, sysUser);
        if (!StringUtils.isEmpty(sysUser.getCnName())) {
            //姓名查询条件
            final List<SysUser> filter = pageInfo.getList().stream().filter(o -> {
                return o.getCnName().contains(sysUser.getCnName());
            }).collect(Collectors.toList());
            pageInfo.setList(filterByRole(sysUser, filter));
        }else if (!StringUtils.isEmpty(sysUser.getRoleIdsStr())) {
            //角色查询条件
            final List<SysUser> filter = pageInfo.getList().stream().filter(o -> {
                return sysUser.getRoleIdsStr().contains(String.valueOf(o.getRoleId()));
            }).collect(Collectors.toList());
            pageInfo.setList(filterByRole(sysUser, filter));
        }else{
            pageInfo.setList(filterByRole(sysUser, pageInfo.getList()));
        }
        return pageInfo;
    }

    @Override
    public ResultVO findList (SysUser sysUser) {
        List<SysUser> list = service.findList(sysUser);
        return success(filterByRole(sysUser, list));
    }

    private List<SysUser> filterByRole (SysUser sysUser, List<SysUser> list) {
        if (sysUser.getRoleId() != null) {
            SysUserRole param = new SysUserRole();
            param.setRoleId(sysUser.getRoleId());
            final List<SysUserRole> userIds = sysUserRoleService.findList(param);
            Set<Integer> userIdSet = new HashSet<>();
            userIds.stream().forEach(x -> {
                userIdSet.add(x.getUserId());
            });

            List<SysUser> result = list.stream().filter(x -> {
                return userIdSet.contains(x.getId());
            }).collect(Collectors.toList());
            return result;
        }
        return list;
    }

    /**
     * 修改用户信息
     */
    @PostMapping("modify")
    public ResultVO editUser (@RequestBody SysUser sysUser) {
        if(sysUser.getNewPassword() != null){
            //修改密码了
            sysUserExtService.save(sysUser);
        }else{
            sysUserExtService.modify(sysUser);
        }
        return success();
    }

    @PostMapping("uni/reg")
    public ResultVO uniReg (@RequestBody SysUser sysUser) {
        if(!SmsUtils.isValid(sysUser.getMobile(),sysUser.getSmsCode())){
            return warn("验证码不正确");
        }
        sysUser.setUsername(sysUser.getMobile());
        sysUserExtService.save(sysUser);

        sysUser.getRoleIds().add(sysUser.getRoleId());
        return success(sysUser);
    }

    @Override
    public ResultVO save (SysUser sysUser) {
        //微信绑定
        if (sysUser.getId() == null && !StringUtils.isEmpty(sysUser.getOpenid())) {
            System.out.println("开始绑定："+sysUser.getOpenid());
            SysUser p = new SysUser();
            p.setOpenid(sysUser.getOpenid());
            final List<SysUser> l = service.findList(p);
            if (l != null && !l.isEmpty()) {
                return success(l.get(0)); //已经绑定过了
            }
        }

        if(sysUser.getId() != null && sysUser.getNewPassword()!=null){ //修改个人信息的时候，处理一下密码，如果为空，则不更新
            sysUser.setPassword(sysUser.getNewPassword());
        }

        sysUserExtService.save(sysUser);

//        sysUser.getRoleIds().add(sysUser.getRoleId());

        SysUserRole userRole = new SysUserRole();
        userRole.setUserId(sysUser.getId());
        sysUserRoleService.deleteByUserId(userRole);
        userRole.setRoleId(sysUser.getRoleId());
        sysUserRoleService.save(userRole);

        return success(sysUser);
    }

    @Override
    public ResultVO delete (SysUser sysUser) {
        service.delete(sysUser);
        SysUserRole userRole = new SysUserRole();
        userRole.setUserId(sysUser.getId());
        sysUserRoleService.deleteByUserId(userRole);
        return success();
    }

}
