package com.oscar;

import com.oscar.base.config.ConfigSysProperties;
import com.oscar.gen.CodeGen;
import com.oscar.gen.CodeGenController;
import com.oscar.gen.Option;
import com.oscar.gen.express.Express;
import com.oscar.gen.vue.Vue;
import com.oscar.sys.user.SysUser;
import com.oscar.sys.user.SysUserMapper;
import freemarker.template.TemplateException;
import lombok.extern.slf4j.Slf4j;
import org.junit.Assert;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

@Slf4j
public class MybatisJpaApplicationTests extends SpringbootUnitTests {

    @Autowired
    CodeGenController codeGenController;
    @Autowired
    ConfigSysProperties config;
    @Value("${spring.application.name}")
    private String appName;

    //引入spring-boot-starter-test即可
    @Test
    public void code() throws TemplateException {
        config.setProjectRootDir("D:/IdeaProjects/private_job/2020");

        System.out.println("=========================================");
        String tableDesc = "意见反馈";
        String tableName = "tb_cms_feedback";

        appName = "vue_school_action";
        config.setPackageModuleName("action");

        boolean hasJava = false;
        boolean hasVue = true;
        boolean hasExpress = false;
        System.out.println("=========================================");


        String dir = config.getProjectRootDir()+"/";
        String moduleName = config.getPackageModuleName();
        String dbSchema = appName;

        Option option = new Option();
        option.setJava(hasJava);
        option.setExpress(hasExpress);
        option.setVue(hasVue);
        option.setVux(false);
        option.setWechat(false);
        option.setAndroid(false);

        CodeGen codeGen = new CodeGen();
        codeGen.setRoot(dir+dbSchema);
        codeGen.setModuleName(moduleName);
        codeGen.setSchema(dbSchema);
        codeGen.setTableName(tableName);

        codeGen.setPackageName("com.oscar."+moduleName);
        codeGen.setProjectPath(codeGen.getRoot()+"/java/");

        Vue vue = new Vue();
        vue.setFilePath(codeGen.getRoot()+"/web/src/page/"+moduleName+"/");
        vue.setTitle(tableDesc);
        vue.setFileName(tableName);
        vue.setControllerPath("/"+tableName.replaceAll("_","/"));
        vue.setExtend(false);
        vue.setDelete(true);
        vue.setAdd(true);
        vue.setUpdate(true);
        vue.setDetail(false);

        Express express = new Express();
        express.setRoutePath(codeGen.getRoot()+"/node/routes/");
        express.setSqlMapperPath(codeGen.getRoot()+"/node/server/mysql/mapper/tb/");
        codeGen.setExpress(express);

        codeGen.setCss("background:#14a0b1db;color:#ffffff;btnBackground:#a3e1e0d6;btnText:#14a0b1;");
        codeGen.setVue(vue);
        codeGen.setOption(option);
        codeGenController.save(codeGen);
    }

    @Test
    public void test1(){
        System.out.println("当前线程：" + Thread.currentThread().getName());
        System.out.println("工程根目录："+System.getProperty("user.dir"));
        System.out.println("CPUs："+Runtime.getRuntime().availableProcessors());
        System.out.println("10进制转16进制：Ox"+Integer.toHexString(3192));
        System.out.println("10进制转2进制：Ox"+Integer.toBinaryString(56));
        Assert.assertEquals(10,100);
    }
    @Test
    public void test2(){
        try {
            System.out.println(5 / 0);
        } catch (Exception e) {
            System.out.println("e.getMessage:" + e.getMessage());

            StackTraceElement[] elements = e.getStackTrace();
            for (StackTraceElement element : elements) {
                System.out.println("getClassName:" + element.getClassName());
                System.out.println("getFileName:" + element.getFileName());
                System.out.println("getLineNumber:" + element.getLineNumber());
                System.out.println("getMethodName:" + element.getMethodName());
                break;
            }
        }
    }



}
