<template>
    <div>
        <x-header class="oscar-bg">${vux.title}</x-header>

        <div v-show="detail.form">
            <group>
                <#list columns as c>
                <#if c.field?? && c.field == "id">
                <#elseif c.dataType?? && c.dataType == "Date">
                <datetime title="${c.columnComment}" v-model="formData.${c.field}"></datetime>
                <#elseif c.dataType?? && c.dataType == "Integer">
                <x-number title="${c.columnComment}" v-model="formData.${c.field}"></x-number>
                <#elseif c.field?? && c.field == "remarks">
                <x-textarea title="${c.columnComment}" :max="200" :height="180" v-model="formData.${c.field}"></x-textarea>
                <#elseif c.field?? && c.field == "imgUrl">
                <#else>
                <x-input title="${c.columnComment}" v-model="formData.${c.field}"></x-input>
                </#if>
                </#list>
                <x-input title="提交人" v-model="formData.username" readonly></x-input>
            </group>

            <div style="padding:15px;">
                <x-button @click.native="onSubmit" type="primary" class="oscar-bg">提交</x-button>
            </div>
        </div>

        <div v-show="!detail.form">

            <flexbox orient="vertical" justify="center">
                <flexbox-item>
                    <div class="flex-demo">
                        <div class="c-title">{{detail.name}}</div>
                        <img :src="baseUrl+'/'+ detail.imgUrl">
                        <div class="c-remarks">{{detail.remarks}}</div>
                        <div class="c-dt">{{detail.createTimeStr}}</div>
                    </div>
                </flexbox-item>
            </flexbox>

<#if vux.hasFFCP == true>
            <flexbox>
                <flexbox-item><div class="flex-abcd" @click="doAction('favorite')"><img src="/static/style/mobile/icon/follow.png">收藏</div></flexbox-item>
                <flexbox-item><div class="flex-abcd" @click="doAction('forward')"><img src="/static/style/mobile/icon/forward.png">转发</div></flexbox-item>
                <flexbox-item><div class="flex-abcd" @click="doAction('comment')"><img src="/static/style/mobile/icon/comment.png">评论</div></flexbox-item>
                <flexbox-item><div class="flex-abcd" @click="doAction('praise')"><img src="/static/style/mobile/icon/praise.png">点赞</div></flexbox-item>
            </flexbox>
</#if>
            <div style="height:10px"></div>
        </div>
    </div>
</template>

<script>
    import {Flexbox, FlexboxItem,XHeader, Group, XInput,XNumber, XButton,XTextarea,Toast,Datetime, Tabbar, TabbarItem} from 'vux'
    import {save, getList} from '@/api'
    import {baseUrl, baseImgPath} from '@/config/env'
    import {mapActions, mapState} from 'vuex'

    export default {
        components: {
            Flexbox, FlexboxItem,XHeader, Group, XInput,XNumber,XButton,XTextarea,Toast,Datetime, Tabbar, TabbarItem
        },
        data() {
            return {
                detail: {
                    form: true,
                },
                formData:{
                    url: '${vue.controllerPath}'
                },
                baseUrl,
            }
        },
        computed: {
            ...mapState(['userInfoState'])
        },
        created() {
            this.dataInit()
        },
        methods: {
            ...mapActions(['getUserAction']),
            dataInit() {
                if(this.$route.query.id){
                    this.detail = this.$route.query
                }
            },
            async onSubmit(){
                let that = this
                let res = await save(this.formData)
                if(res.code == 0){
                    this.$vux.toast.show({
                        text: '提交成功',
                        onHide(){
                            that.$router.back()
                        }
                    })
                }
            },
            doAction(type){
                let that = this
                if(type == 'favorite'){
                    this.$vux.toast.show({
                        text: '收藏成功',
                    })
                }else if(type =='comment'){
                    this.$router.push({
                        path:'/m/comment',
                        query:{
                            articleId:this.detail.id,
                            title:this.detail.name,
                            userId:this.userInfoState.id,
                            cnName:this.userInfoState.cnName,
                            source:'tb_favorite'  // need  modify
                        }
                    })
                }
            },

        },
    }
</script>

<style lang="less" scoped>
    .flex-abcd{
        text-align: center;
        padding: 16px;
        font-size: 14px;
    }
    .flex-abcd img{
        width: 14px;
        margin-right: 5px;
    }
    .flex-demo {
        text-align: center;
        padding: 16px;
    }
    .flex-demo img{
        width: 100%;
    }
    .c-title{
        font-weight: bold;margin-bottom: 20px;font-size:36px;
    }
    .c-remarks{
        font-size:16px;margin-top: 10px;
    }
    .c-dt{
        color:gray;float:right;font-size:16px;margin: 10px;
    }
</style>