package com.oscar.module.crawl;

import com.oscar.base.util.FileUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * Desc :
 *
 * @author wl
 * Date:    2019/1/9 8:03
 * @version 1.0
 */
public abstract class Website {
    protected Logger logger = LoggerFactory.getLogger(this.getClass());

    private static final String desktop = "C:/Users/oscar/Desktop/";

    private String url;
    private Document doc;
    private String referrer = "";
    private String accept = "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8";
    private String contentType = "text/html";
    private Map<String,String> cookies = new HashMap<>();
    private Map<String, String> queryParam = new HashMap<>();

    protected Website setReferrer(String referrer){
        this.referrer = referrer;
        return this;
    }
    protected Website addCookie(String key,String value){
        cookies.put(key, value);
        return this;
    }
    protected Website addQueryParam(String key,String value){
        queryParam.put(key, value);
        return this;
    }

    private void parseUrl (Object... placeholder) {
        for (int i = 0; i < placeholder.length; i++) {
            url = url.replaceFirst("\\{" + i + "\\}", String.valueOf(placeholder[i]));
        }
    }

    protected abstract void successProcess(Document doc);
    protected void errorProcess(Exception e){
        e.printStackTrace();
    }
    protected Website setUrl (String url){
        this.url = url;
        return this;
    }
    protected Website setAccept (String accept){
        this.accept = accept;
        return this;
    }
    protected Website setJsonContentType(){
        this.contentType = "application/json";
        return this;
    }

    public Website start (Object... placeholder) {
        logger.info("start is {}",placeholder);

        parseUrl(placeholder);

        Map<String, String> header = new HashMap<>();
        header.put("Host", "http://info.bet007.com");
        header.put("Accept", this.accept);
        header.put("Accept-Language", "zh-CN,zh;q=0.9");
        header.put("Connection", "keep-alive");
        header.put("Content-Type",this.contentType);

        try {
            doc = Jsoup.connect(url)
                    .timeout(60000)
                    .data(queryParam)
                    .headers(header)
                    .userAgent(UA.CHROME)
                    .referrer(referrer)
                    .cookies(cookies)
                    .ignoreContentType(true)
                    .get();
            successProcess(doc);
        } catch (Exception e) {
            errorProcess(e);
        }
        return this;
    }

    /**
     * 查询唯一的元素
     * @param id
     * @return
     */
    protected Element getById(String id){
        Elements elements = doc.select(id);
        return elements.first();
    }

    private void test(){
        //正则
        doc.getElementsByAttributeValueMatching("href","/view/(\\w)+(\\.)html");
        doc.select("span.qyBand a");


        final Elements select = doc.select("#wang");
        final Element first = select.first();//select.get(0)
        System.out.println(first.ownText());
        System.out.println(first.outerHtml());
        System.out.println(first.html());
        System.out.println(first.attr("id"));
    }

    protected String getJson(){
        return doc.body().html();
    }

    void savePage(){
        FileUtils.save(doc.html(),new File(desktop+"page.html"));
    }
}
