package com.oscar.module.test.kafka.stream;

import org.apache.kafka.streams.KeyValue;
import org.apache.kafka.streams.processor.*;
import org.apache.kafka.streams.state.KeyValueIterator;
import org.apache.kafka.streams.state.KeyValueStore;

import java.time.Duration;
import java.util.stream.Stream;

public class SourceProcessor implements ProcessorSupplier<String, String> {

    @Override
    public Processor<String, String> get () {
        return new Processor<String, String>() {
            private ProcessorContext context;
            private KeyValueStore<String, Integer> kvStore;

            @Override
            public void init (ProcessorContext context) {
                this.context = context;
                this.kvStore = (KeyValueStore) context.getStateStore("ss_counts");
                //7s周期性调度
                final Cancellable schedule = this.context.schedule(Duration.ofMillis(7000), PunctuationType.WALL_CLOCK_TIME, timestamp -> {
                    KeyValueIterator it = kvStore.all();
                    while (it.hasNext()) {
                        KeyValue entry = (KeyValue) it.next();
                        //转发到downstream processors，即本例子中的PrintProcessor
                        context.forward(entry.key, entry.value.toString());
                    }
                    context.commit();//提交处理进度
                });
            }

            /**
             * 当有消息到达时候调用
             * @param key
             * @param value 记录
             */
            @Override
            public void process (String key, String value) {
                Stream.of(value.toLowerCase().split(" ")).forEach(word->{
                    Integer wordCount = this.kvStore.get(word);
                    if (wordCount == null) {
                        this.kvStore.put(word, 1);
                    } else {
                        this.kvStore.put(word, wordCount + 1);
                    }
                });
//                context.commit();//提交处理进度
            }

            @Override
            public void close () {
                this.kvStore.close();
            }
        };
    }

}
