package com.oscar.module.test.zk;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.oscar.base.AbstractController;
import com.oscar.base.util.FileUtils;
import com.oscar.base.vo.ResultVO;
import org.springframework.web.bind.annotation.*;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import java.io.IOException;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/zk")
public class ZookeeperController extends AbstractController{
    private ZkClient zkClient = null;

    @PostConstruct
    public void init () {
    }

    @GetMapping("ping")
    public boolean ping(){
        return !zkClient.isClose();
    }

    @GetMapping("connect")
    public void connect(String hosts){
        if(zkClient != null && !zkClient.isClose())
            return;

        zkClient = new ZkClient();
        try {
            zkClient.connect(hosts);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @GetMapping("disConnect")
    public void disConnect(String hosts) throws InterruptedException {
        if(zkClient != null){
            zkClient.close();
        }
    }

    @PreDestroy
    public void destroy () {
        try {
            zkClient.close();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    /**
     * GET http://localhost:8080/zk/ls?path=/brokers
     */
    @GetMapping("ls")
    public Map<String, Object> ls (String path) throws Exception {

        final Map<String, Object> map = zkClient.getData(path, true);
        final List<String> children = zkClient.ls(path);
        map.put("children",children);
        return map;
    }

    @GetMapping("location/{pathId}")
    public JSONObject location109 (@PathVariable String pathId) throws IOException {
        return getJsonObject(FileUtils.readResources("txt/" + pathId));
    }

    private JSONObject getJsonObject (String response) {
        JSONObject jsonObject = JSON.parseObject(response);
        return jsonObject;
    }

    @PostMapping()
    public ResultVO save (@RequestBody ZNode zNode) {
        try {
            if(zkClient.exists(zNode.getPath())){
                zkClient.setData(zNode.getPath(),zNode.getData());
            }else{
                zkClient.create(zNode);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return error(e);
        }
        return success();
    }

}
